#######################################################################
#######  Combination of Uniform and Multinomial (CUM Model)   #########     
#######################################################################
# Authors: Marica Manisera and Paola Zuccolotto
# February 2021
 
# This entire File Header must accompany all files using any portion, in whole or in part,
# of this Source Code. If you wish to distribute any portion of the proprietary Source Code,
# you must first obtain written permission from the Author.
                                                                       
# If you use this program or part of it for applied or research projects, please inform the
# Authors via e-mail to allow updating references' file (marica.manisera@unibs.it, paola.zuccolotto@unibs.it)
# When using this software, please correctly acknowledge it. 

# Main reference: 
# 
# 
                                                                     
# This program is freely released without any responsibility for Authors. 




#######################################################################
require(ggtern)
require(mc2d)
require(rootSolve)
require(maxLik)


#######################################################################
##############             data simulation            #################
#######################################################################
# inputs:
# param 	     = parameter values c(pai,xiD,xiU)
# Nsim 		     = sample size

# output: a sample of simulated data 

simul7 <- function(param,Nsim){
pai <- param[1]
xiD <- param[2]
xiU <- param[3]

probs <- array(NA,7)

	probs[1] <- dmultinom(c(3,0,0),3,prob=c(xiD,xiU,(1-xiD-xiU)))
	probs[2] <- dmultinom(c(2,0,1),3,prob=c(xiD,xiU,(1-xiD-xiU)))
	probs[3] <- dmultinom(c(2,1,0),3,prob=c(xiD,xiU,(1-xiD-xiU))) +
			dmultinom(c(1,0,2),3,prob=c(xiD,xiU,(1-xiD-xiU)))
	probs[4] <- dmultinom(c(1,1,1),3,prob=c(xiD,xiU,(1-xiD-xiU))) + 
			dmultinom(c(0,0,3),3,prob=c(xiD,xiU,(1-xiD-xiU)))
	probs[5] <- dmultinom(c(1,2,0),3,prob=c(xiD,xiU,(1-xiD-xiU))) +
			dmultinom(c(0,1,2),3,prob=c(xiD,xiU,(1-xiD-xiU)))
	probs[6] <- dmultinom(c(0,2,1),3,prob=c(xiD,xiU,(1-xiD-xiU)))
	probs[7] <- dmultinom(c(0,3,0),3,prob=c(xiD,xiU,(1-xiD-xiU)))

Probs <- c(0,cumsum(probs))
uW <- runif(Nsim)
rW <- array(NA,Nsim)
for(i in 1:Nsim){
rW[i] <- max(which(Probs<=uW[i]))
}

uU <- runif(Nsim)
rU <- sample.int(7, size = Nsim, replace = T)

R <- rW
for(i in 1:Nsim){
if(uU[i] >= pai) R[i] <- rU[i]
}

R
}

#######################################################################
##############              fitted values             #################
#######################################################################
# inputs:
# param 	     = parameter values c(pai,xiD,xiU)
# m		     = numer of categories (must be equal to 7)

# output: values of the variables W and fitted values 

thfr7 <- function(param,m=7) {
pai <- param[1]
xiD <- param[2]
xiU <- param[3]

	probs <- matrix(NA,7,1)

	probs[1] <- dmultinom(c(3,0,0),3,prob=c(xiD,xiU,1-xiD-xiU))
	probs[2] <- dmultinom(c(2,0,1),3,prob=c(xiD,xiU,1-xiD-xiU))
	probs[3] <- dmultinom(c(2,1,0),3,prob=c(xiD,xiU,1-xiD-xiU)) +
			dmultinom(c(1,0,2),3,prob=c(xiD,xiU,1-xiD-xiU))
	probs[4] <- dmultinom(c(1,1,1),3,prob=c(xiD,xiU,1-xiD-xiU)) + 
			dmultinom(c(0,0,3),3,prob=c(xiD,xiU,1-xiD-xiU))
	probs[5] <- dmultinom(c(1,2,0),3,prob=c(xiD,xiU,1-xiD-xiU)) +
			dmultinom(c(0,1,2),3,prob=c(xiD,xiU,1-xiD-xiU))
	probs[6] <- dmultinom(c(0,2,1),3,prob=c(xiD,xiU,1-xiD-xiU))
	probs[7] <- dmultinom(c(0,3,0),3,prob=c(xiD,xiU,1-xiD-xiU))


tfW <- probs

tf <- pai*tfW + (1-pai)*(1/m)
list("W"=tfW,"fit"=tf)
}


#######################################################################
##############         diss index - CUM model         #################
#######################################################################
# inputs:
# rd 		     = observed ratings
# paramd	     = parameter values c(pai,xiD,xiU)
# m		     = numer of categories (must be equal to 7)

# output: value of the dissimilarity index for a CUM model with given parameters


dissCUM <- function(rd,paramd,m=7) {
	
	tabr<-as.matrix(table(rd))
		if(length(tabr) < m){
		rr<-matrix(0,m,1)
		rr[as.integer(row.names(tabr)),]<-tabr
		tabr<-rr
		}
	
	fr <- tabr/sum(tabr)
	diss <- 1/2*sum(abs(fr-thfr7(param=paramd)$fit))
	diss
}

#######################################################################
##############       diss index - generic model       #################
#######################################################################
# inputs:
# rd 		     = observed ratings
# thfd   	     = fitted frequencies (an array)
# m		     = numer of categories

# output: value of the dissimilarity index for a generic model


diss <- function(rd,thfd,m=7) {
	
	tabr<-as.matrix(table(rd))
		if(length(tabr) < m){
		rr<-matrix(0,m,1)
		rr[as.integer(row.names(tabr)),]<-tabr
		tabr<-rr
		}
	
	fr <- tabr/sum(tabr)
	diss <- 1/2*sum(abs(fr-thfd))
	diss
}


#######################################################################
#########  OBSERVED VS. FITTED FREQUENCIES PLOT - CUM model  ##########
#######################################################################
# inputs:
# rp 		     = observed ratings
# paramp	     = parameter values c(pai,xiD,xiU) of the CUM model
# m		     = numer of categories (must be equal to 7)

# output: 
# the observed vs. fitted frequencies plot for a CUM model with given parameters

CUMplot <- function(rp,paramp,m=7){

		tabr<-as.matrix(table(rp))
		if(length(tabr) < m){
		rr<-matrix(0,m,1)
		rr[as.integer(row.names(tabr)),]<-tabr
		tabr<-rr
		}

maxplot <- max(tabr/sum(tabr),thfr7(param=paramp)$fit)*1.3

plot(tabr/sum(tabr),axes=FALSE,main=paste("CUM model - diss = ",round(dissCUM(rd=rp,paramd=paramp,m=m),4),sep=""),
	xlab='Ratings',pch=16,cex.lab=0.8,
	ylab='Observed relative frequencies (dots) and fitted probabilities (circles)',ylim=c(0,maxplot))

box()
axis(1,1:m)
axis(2)
points(thfr7(param=paramp)$fit,type='b',lty='dashed',cex=1.4)
abline(c(0,0),c(0,0))


}

#######################################################################
#######  OBSERVED VS. FITTED FREQUENCIES PLOT - generic model  ########
#######################################################################
# inputs:
# rp 		     = observed ratings
# thfp   	     = fitted frequencies (an array)
# m		     = numer of categories
# name		     = name of the model under investigation

# output: 
# the observed vs. fitted frequencies plot for a generic model whose fitted frequencies are given in input

OFplot <- function(rp,thfp,m=7,name="Alternative Model"){

		tabr<-as.matrix(table(rp))
		if(length(tabr) < m){
		rr<-matrix(0,m,1)
		rr[as.integer(row.names(tabr)),]<-tabr
		tabr<-rr
		}

maxplot <- max(tabr/sum(tabr),thfp)*1.3

plot(tabr/sum(tabr),axes=FALSE,main=paste(name, " - diss = ",round(diss(rd=rp,thfd=thfp,m=m),4),sep=""),
	xlab='Ratings',pch=16,cex.lab=0.8,
	ylab='Observed relative frequencies (dots) and fitted probabilities (circles)',ylim=c(0,maxplot))

box()
axis(1,1:m)
axis(2)
points(thfp,type='b',lty='dashed',cex=1.4)
abline(c(0,0),c(0,0))


}

#######################################################################
##############           Triangular plot              #################
#######################################################################
# inputs:
# nmod		     = number of estimated models to represent in the same triangle
# param  	     = parameter values: if nmod=1, use the syntax c(pai,xiD,xiU);
#		       if nmod>1 a matrix with nmod rows and columns pai, xiD, xiU
# col.p	             = color for the point(s)
# a		     = transparency

# output: 
# the parameters' triangulat plot

triCUMplot <- function(nmod=1,param,col.p='black',a=0.5) {

if(nmod==1){
pai <- param[1]
xiD <- param[2]
xiU <- param[3]
xi <- 1 - xiD - xiU
}

if(nmod>1){
pai <- param[,1]
xiD <- param[,2]
xiU <- param[,3]
xi <- 1 - xiD - xiU
}

df <- data.frame(xiD = xiD, xiU = xiU, xi = xi, unc=1-pai)
print(str(df))
labFnc <- function(x,digits=3) format(round(unique(x),digits),digits=digits)

tern <- ggtern(df, aes(xiD,xiU,xi)) +
  geom_point(aes(size = unc), alpha = a, col=col.p) +
  scale_size_area(breaks=seq(0,1,0.2), limits=c(0,1)) +
  scale_T_continuous(breaks=unique(df$xiU),labels=labFnc(df$xiU)) +
  scale_L_continuous(breaks=unique(df$xiD),labels=labFnc(df$xiD)) +
  scale_R_continuous(breaks=unique(df$xi),labels=labFnc(df$xi)) +
  labs(x = "Down", y = "Up", z = "Stay", size = "Uncertainty")+
  theme_rgbw() +
  theme_nogrid() +
  geom_Lmark(colour='darkblue') +
  geom_Tmark(colour='darkred') +
  geom_Rmark(colour='darkgreen') +
  theme(legend.key = element_rect(fill="white"),
        tern.axis.line   = element_line(size=1))+
  geom_text(data=df, aes(xiD,xiU,label = round(unc,digits=3)),hjust=1.5,size=3, col=col.p)

return(tern)
}

#######################################################################
##############     add points to triangular plot      #################
#######################################################################
# inputs:
# tern		     = a ggtern plot created with triCUMplot
# param  	     = parameter values to add: a data frame with columns pai, xiD, xiU
# col.p	             = color for the point(s)
# a		     = transparency

# output: 
# the tern plot with the added points

addpoints <- function(tern,param,col.p='black',a=0.2) {
names(param) <- c("pai","xiD","xiU")

tern1 <- tern +
   geom_point(data=param, aes(x=xiD, y=xiU, z=1-xiD-xiU, size = 1 - pai), alpha = a, col=col.p) 

return(tern1)

}


#######################################################################
##############                EM algorithm            #################
#######################################################################
# inputs:
# init	 	     = starting values for c(pai,xiD,xiU)
# R 		     = observed ratings 
# toler		     = tolerance for convergence
# m		     = numer of categories (must be equal to 7)
# plot		     = produce OBSERVED VS. FITTED FREQUENCIES PLOT
# max.iter	     = maximum number of iterations

# output: model estimation with EM algorithm

EM7 <- function(init=c(0.5,0.3,0.3),R,toler = 0.00001,m=7,plot=TRUE,max.iter=700){
pai <- init[1]
xiD <- init[2]
xiU <- init[3]

	tabr<-as.matrix(table(R))
		if(length(tabr) < m){
		rr<-matrix(0,m,1)
		rr[as.integer(row.names(tabr)),]<-tabr
		tabr<-rr
		}


k <- 0
diffpai <- 1
diffxiD <- 1
diffxiU <- 1

while (diffpai > toler | diffxiD > toler | diffxiU > toler){
if(k <= max.iter){
k <- k+1
W <- thfr7(c(pai[k],xiD[k],xiU[k]))$W
pt <- thfr7(c(pai[k],xiD[k],xiU[k]))$fit

########################################
################################# E-step

tau <- (pai[k]*W)/pt

########################################
################################# M-step

pai <- c(pai,weighted.mean(tau,tabr))		

funct <- function(x){
# to ensure values inside the parameter space
xD <- exp(x[1])/(1+exp(x[1])+exp(x[2]))
xU <- exp(x[2])/(1+exp(x[1])+exp(x[2]))

if(is.na(xD)) xD <- 0.5
if(is.na(xU)) xU <- 0.5

WD <- c(dmultinom(c(2,0,0),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))),
	dmultinom(c(1,0,1),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))) - dmultinom(c(2,0,0),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))),
	dmultinom(c(1,1,0),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))) + dmultinom(c(0,0,2),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))) - dmultinom(c(1,0,1),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))),
	
	dmultinom(c(0,1,1),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))) - dmultinom(c(1,1,0),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))) - dmultinom(c(0,0,2),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))),
	dmultinom(c(0,2,0),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))) - dmultinom(c(0,1,1),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))),
	-dmultinom(c(0,2,0),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))),
	0)

WU <- c(0,
	-dmultinom(c(2,0,0),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))),
	dmultinom(c(2,0,0),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))) - dmultinom(c(1,0,1),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))),

	dmultinom(c(1,0,1),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))) - dmultinom(c(1,1,0),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))) - dmultinom(c(0,0,2),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))),
	dmultinom(c(1,1,0),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))) + dmultinom(c(0,0,2),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))) - dmultinom(c(0,1,1),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))),
	dmultinom(c(0,1,1),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))) - dmultinom(c(0,2,0),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))),
	dmultinom(c(0,2,0),2,prob=c(max(0,xD),max(0,xU),max(0,1-xD-xU))))

fD <- weighted.mean(tau/thfr7(c(pai[k],xD,xU))$W * WD,tabr)
fU <- weighted.mean(tau/thfr7(c(pai[k],xD,xU))$W * WU,tabr)
c(fD=fD,fU=fU)
}


ss <- multiroot(f = funct, start = c(1,1))
xxD <- exp(ss$root[1])/(1+exp(ss$root[1])+exp(ss$root[2]))
xxU <- exp(ss$root[2])/(1+exp(ss$root[1])+exp(ss$root[2]))


xiD <- c(xiD,xxD)
xiU <- c(xiU,xxU)


diffpai <- abs(pai[k+1]-pai[k])
diffxiD <- abs(xiD[k+1]-xiD[k])
diffxiU <- abs(xiU[k+1]-xiU[k])

} # end if(k <= max.iter)

if(k > max.iter){
k <- k-1
diffpai <- 0
diffxiD <- 0
diffxiU <- 0
print("maximum number of iterations reached")
} # end if(k > max.iter)

} # end while (diffpai > toler | diffxiD > toler | diffxiU > toler)

pEM <- pai[k+1]
xDEM <- xiD[k+1]
xUEM <- xiU[k+1]

parxiEM <- c(xDEM,xUEM,1-xDEM-xUEM)
diss <- dissCUM(rd=R,paramd=c(pEM,xDEM,xUEM),m=7)

############# Observed Information Matrix

n1 <- tabr[1]
n2 <- tabr[2]
n3 <- tabr[3]
n4 <- tabr[4]
n5 <- tabr[5]
n6 <- tabr[6]
n7 <- tabr[7]
n <- n1+n2+n3+n4+n5+n6+n7

############# I pi pi

# pr <- thfr7(c(pEM,xDEM,xUEM))
# Ipipi <- n * weighted.mean( ( (pr$W - (1/m))/pr$fit )^2 , tabr)


Ipipi <- 			  n1*((dmultinom(c(3,0,0),3,prob=parxiEM)-(1/m))/
				  (pEM*( dmultinom(c(3,0,0),3,prob=parxiEM)-(1/m) ) +(1/m) ))^2+
				  + n2*((dmultinom(c(2,0,1),3,prob=parxiEM)-(1/m))/
				  (pEM*( dmultinom(c(2,0,1),3,prob=parxiEM)-(1/m) ) +(1/m) ))^2+
				  + n3*((dmultinom(c(1,0,2),3,prob=parxiEM)+
				  +dmultinom(c(2,1,0),3,prob=parxiEM)-(1/m))/
				  (pEM*( dmultinom(c(1,0,2),3,prob=parxiEM)+
				  +dmultinom(c(2,1,0),3,prob=parxiEM)-(1/m) ) +(1/m) ))^2+
				  + n4*((dmultinom(c(0,0,3),3,prob=parxiEM)+
				  +dmultinom(c(1,1,1),3,prob=parxiEM)-(1/m))/
				  (pEM*( dmultinom(c(0,0,3),3,prob=parxiEM)+
				  +dmultinom(c(1,1,1),3,prob=parxiEM)-(1/m) ) +(1/m) ))^2+
				  + n5*((dmultinom(c(0,1,2),3,prob=parxiEM)+
				  +dmultinom(c(1,2,0),3,prob=parxiEM)-(1/m))/
				  (pEM*( dmultinom(c(0,1,2),3,prob=parxiEM)+
				  +dmultinom(c(1,2,0),3,prob=parxiEM)-(1/m) ) +(1/m) ))^2+
				  + n6*((dmultinom(c(0,2,1),3,prob=parxiEM)-(1/m))/
				  (pEM*( dmultinom(c(0,2,1),3,prob=parxiEM)-(1/m) ) +(1/m) ))^2+
				  + n7*((dmultinom(c(0,3,0),3,prob=parxiEM)-(1/m))/
				  (pEM*( dmultinom(c(0,3,0),3,prob=parxiEM)-(1/m) ) +(1/m) ))^2
				  

############# I xiD xiD
IxiDxiD <- pEM * (
				  pEM*n1* ((3*dmultinom(c(2,0,0),2,prob=parxiEM))/
				  (pEM*( dmultinom(c(3,0,0),3,prob=parxiEM)-(1/m) ) +(1/m) ))^2
				  - n1* ((6*xDEM)/
				  (pEM*( dmultinom(c(3,0,0),3,prob=parxiEM)-(1/m) ) +(1/m) ))
				  + pEM*n2* (( 3*(dmultinom(c(1,0,1),2,prob=parxiEM)-
				  dmultinom(c(2,0,0),2,prob=parxiEM)))/
				  (pEM*( dmultinom(c(2,0,1),3,prob=parxiEM)-(1/m) ) +(1/m) ))^2
				  - n2* ((6*(1-xUEM-3*xDEM)/
				  (pEM*( dmultinom(c(2,0,1),3,prob=parxiEM)-(1/m) ) +(1/m) )))
				  + pEM*n3* (( 3*(dmultinom(c(1,1,0),2,prob=parxiEM)+
				  dmultinom(c(0,0,2),2,prob=parxiEM) -
				  dmultinom(c(1,0,1),2,prob=parxiEM)))/
				  (pEM*( dmultinom(c(1,0,2),3,prob=parxiEM)+dmultinom(c(2,1,0),3,prob=parxiEM)
				  -(1/m) ) +(1/m) ))^2
				  + n3* ((6*(2-3*xDEM-3*xUEM)/
				  (pEM*( dmultinom(c(1,0,2),3,prob=parxiEM)+dmultinom(c(2,1,0),3,prob=parxiEM)-
				  (1/m) ) +(1/m) )))
				  + pEM* n4* (( 3*(dmultinom(c(0,1,1),2,prob=parxiEM)-
				  dmultinom(c(1,1,0),2,prob=parxiEM) -
				  dmultinom(c(0,0,2),2,prob=parxiEM)))/
				  (pEM*( dmultinom(c(0,0,3),3,prob=parxiEM)+dmultinom(c(1,1,1),3,prob=parxiEM)
				  -(1/m) ) +(1/m) ))^2
				  - n4* ((6*(1-xDEM-3*xUEM)/
				  (pEM*( dmultinom(c(0,0,3),3,prob=parxiEM)+dmultinom(c(1,1,1),3,prob=parxiEM)-
				  (1/m) ) +(1/m) )))
				  + pEM*n5* (( 3*(dmultinom(c(0,2,0),2,prob=parxiEM)-
				  dmultinom(c(0,1,1),2,prob=parxiEM) ))/
				  (pEM*( dmultinom(c(0,1,2),3,prob=parxiEM)+dmultinom(c(1,2,0),3,prob=parxiEM)
				  -(1/m) ) +(1/m) ))^2
				  - n5* ((6*xUEM)/
				  (pEM*( dmultinom(c(0,1,2),3,prob=parxiEM)+dmultinom(c(1,2,0),3,prob=parxiEM)-
				  (1/m) ) +(1/m) ))
				  + pEM*n6* ((-3*dmultinom(c(0,2,0),2,prob=parxiEM))/
				  (pEM*( dmultinom(c(0,2,1),3,prob=parxiEM)-(1/m) ) +(1/m) ))^2
				  )

############# I xiU xiU
IxiUxiU <- pEM * (pEM*n2* ((-3*dmultinom(c(2,0,0),2,prob=parxiEM))/
				  (pEM*( dmultinom(c(2,0,1),3,prob=parxiEM)-(1/m) ) +(1/m) ))^2
				  + pEM*n3* (( 3*(dmultinom(c(2,0,0),2,prob=parxiEM)-
				  dmultinom(c(1,0,1),2,prob=parxiEM) ))/
				  (pEM*( dmultinom(c(1,0,2),3,prob=parxiEM)+dmultinom(c(2,1,0),3,prob=parxiEM)
				  -(1/m) ) +(1/m) ))^2
				  - n3* ((6*xDEM)/
				  (pEM*( dmultinom(c(1,0,2),3,prob=parxiEM)+dmultinom(c(2,1,0),3,prob=parxiEM)-
				  (1/m) ) +(1/m) ))
				  + pEM* n4* (( 3*(dmultinom(c(1,0,1),2,prob=parxiEM)-
				  dmultinom(c(1,1,0),2,prob=parxiEM) -
				  dmultinom(c(0,0,2),2,prob=parxiEM)))/
				  (pEM*( dmultinom(c(0,0,3),3,prob=parxiEM)+dmultinom(c(1,1,1),3,prob=parxiEM)
				  -(1/m) ) +(1/m) ))^2
				  - n4* ((6*(1-xUEM-3*xDEM)/
				  (pEM*( dmultinom(c(0,0,3),3,prob=parxiEM)+dmultinom(c(1,1,1),3,prob=parxiEM)-
				  (1/m) ) +(1/m) )))
				  + pEM*n5* (( 3*(dmultinom(c(1,1,0),2,prob=parxiEM)+
				  dmultinom(c(0,0,2),2,prob=parxiEM) -
				  dmultinom(c(0,1,1),2,prob=parxiEM)))/
				  (pEM*( dmultinom(c(0,1,2),3,prob=parxiEM)+dmultinom(c(1,2,0),3,prob=parxiEM)
				  -(1/m) ) +(1/m) ))^2
				  + n5* ((6*(2-3*xDEM-3*xUEM)/
				  (pEM*( dmultinom(c(0,1,2),3,prob=parxiEM)+dmultinom(c(1,2,0),3,prob=parxiEM)-
				  (1/m) ) +(1/m) )))
				  + pEM*n6* (( 3*(dmultinom(c(0,1,1),2,prob=parxiEM)-
				  dmultinom(c(0,2,0),2,prob=parxiEM)))/
				  (pEM*( dmultinom(c(0,2,1),3,prob=parxiEM)-(1/m) ) +(1/m) ))^2
				  - n6* ((6*(1-xDEM-3*xUEM)/
				  (pEM*( dmultinom(c(0,2,1),3,prob=parxiEM)-(1/m) ) +(1/m) )))
				   +pEM*n7* ((3*dmultinom(c(0,2,0),2,prob=parxiEM))/
				  (pEM*( dmultinom(c(0,3,0),3,prob=parxiEM)-(1/m) ) +(1/m) ))^2
				  - n7* ((6*xUEM)/
				  (pEM*( dmultinom(c(0,3,0),3,prob=parxiEM)-(1/m) ) +(1/m) ))		  
				  )
				  
				  
				  
############# I pi xiD
IpixiD <- 		pEM*n1* (3*dmultinom(c(2,0,0),2,prob=parxiEM)*( dmultinom(c(3,0,0),3,prob=parxiEM)-(1/m) ) ) /
				   (pEM*( dmultinom(c(3,0,0),3,prob=parxiEM)-(1/m) ) +(1/m) )^2 +
				   (- n1)* ((3*dmultinom(c(2,0,0),2,prob=parxiEM))/
				   (pEM*( dmultinom(c(3,0,0),3,prob=parxiEM)-(1/m) ) +(1/m) ))+
				   + pEM*n2* (3* (dmultinom(c(1,0,1),2,prob=parxiEM)-dmultinom(c(2,0,0),2,prob=parxiEM))*( dmultinom(c(2,0,1),3,prob=parxiEM)-(1/m) ) ) /
				   (pEM*( dmultinom(c(2,0,1),3,prob=parxiEM)-(1/m) ) +(1/m) )^2 +
				   (- n2)* (3* (dmultinom(c(1,0,1),2,prob=parxiEM)-dmultinom(c(2,0,0),2,prob=parxiEM)) ) /
				   (pEM*( dmultinom(c(2,0,1),3,prob=parxiEM)-(1/m) ) +(1/m) )+
				   + pEM*n3* (3* (dmultinom(c(1,1,0),2,prob=parxiEM)+dmultinom(c(0,0,2),2,prob=parxiEM)-dmultinom(c(1,0,1),2,prob=parxiEM))*
				   ( dmultinom(c(1,0,2),3,prob=parxiEM)+dmultinom(c(2,1,0),3,prob=parxiEM)-(1/m) ) ) /
				   (pEM*( dmultinom(c(1,0,2),3,prob=parxiEM)+dmultinom(c(2,1,0),3,prob=parxiEM)-(1/m) ) +(1/m) )^2+
				   (- n3)* (3* (dmultinom(c(1,1,0),2,prob=parxiEM)+dmultinom(c(0,0,2),2,prob=parxiEM)-dmultinom(c(1,0,1),2,prob=parxiEM)) ) /
				   (pEM*( dmultinom(c(1,0,2),3,prob=parxiEM)+dmultinom(c(2,1,0),3,prob=parxiEM)-(1/m) ) +(1/m) )+
				   + pEM*n4* (3* (dmultinom(c(0,1,1),2,prob=parxiEM)-dmultinom(c(1,1,0),2,prob=parxiEM)-dmultinom(c(0,0,2),2,prob=parxiEM))*
				   ( dmultinom(c(0,0,3),3,prob=parxiEM)+dmultinom(c(1,1,1),3,prob=parxiEM)-(1/m) ) ) /
				   (pEM*( dmultinom(c(0,0,3),3,prob=parxiEM)+dmultinom(c(1,1,1),3,prob=parxiEM)-(1/m) ) +(1/m) )^2+
				   (- n4)* (3* (dmultinom(c(0,1,1),2,prob=parxiEM)-dmultinom(c(1,1,0),2,prob=parxiEM)-dmultinom(c(0,0,2),2,prob=parxiEM)) ) /
				   (pEM*( dmultinom(c(0,0,3),3,prob=parxiEM)+dmultinom(c(1,1,1),3,prob=parxiEM)-(1/m) ) +(1/m) )+
				   + pEM*n5* (3* (dmultinom(c(0,2,0),2,prob=parxiEM)-dmultinom(c(0,1,1),2,prob=parxiEM))*
				   ( dmultinom(c(0,1,2),3,prob=parxiEM)+dmultinom(c(1,2,0),3,prob=parxiEM)-(1/m) ) ) /
				   (pEM*( dmultinom(c(0,1,2),3,prob=parxiEM)+dmultinom(c(1,2,0),3,prob=parxiEM)-(1/m) ) +(1/m) )^2+
				   (- n5)* (3* (dmultinom(c(0,2,0),2,prob=parxiEM)-dmultinom(c(0,1,1),2,prob=parxiEM)) ) /
				   (pEM*( dmultinom(c(0,1,2),3,prob=parxiEM)+dmultinom(c(1,2,0),3,prob=parxiEM)-(1/m) ) +(1/m) )+
				   + pEM*(-n6)* (3*dmultinom(c(0,2,0),2,prob=parxiEM)*( dmultinom(c(0,2,1),3,prob=parxiEM)-(1/m) ) ) /
				   (pEM*( dmultinom(c(0,2,1),3,prob=parxiEM)-(1/m) ) +(1/m) )^2+
				   + n6* ((3*dmultinom(c(0,2,0),2,prob=parxiEM))/
				   (pEM*( dmultinom(c(0,2,1),3,prob=parxiEM)-(1/m) ) +(1/m) ))		
					
										
					
############# I pi xiU
IpixiU <- pEM*(-n2)* (3*dmultinom(c(2,0,0),2,prob=parxiEM)*( dmultinom(c(2,0,1),3,prob=parxiEM)-(1/m) ) ) /
				   (pEM*( dmultinom(c(2,0,1),3,prob=parxiEM)-(1/m) ) +(1/m) )^2+
				   + n2* ((3*dmultinom(c(2,0,0),2,prob=parxiEM))/
				   (pEM*( dmultinom(c(2,0,1),3,prob=parxiEM)-(1/m) ) +(1/m) ))+
				   + pEM*n3* (3* (dmultinom(c(2,0,0),2,prob=parxiEM)-dmultinom(c(1,0,1),2,prob=parxiEM))*
				   ( dmultinom(c(1,0,2),3,prob=parxiEM)+dmultinom(c(2,1,0),3,prob=parxiEM)-(1/m) ) ) /
					(pEM*( dmultinom(c(1,0,2),3,prob=parxiEM)+dmultinom(c(2,1,0),3,prob=parxiEM)-(1/m) ) +(1/m) )^2+
				   (- n3)* (3* (dmultinom(c(2,0,0),2,prob=parxiEM)-dmultinom(c(1,0,1),2,prob=parxiEM)) ) /
				   (pEM*( dmultinom(c(1,0,2),3,prob=parxiEM)+dmultinom(c(2,1,0),3,prob=parxiEM)-(1/m) ) +(1/m) )+
				   + pEM*n4* (3* (dmultinom(c(1,0,1),2,prob=parxiEM)-dmultinom(c(1,1,0),2,prob=parxiEM)-dmultinom(c(0,0,2),2,prob=parxiEM))*
				   ( dmultinom(c(0,0,3),3,prob=parxiEM)+dmultinom(c(1,1,1),3,prob=parxiEM)-(1/m) ) ) /
				   (pEM*( dmultinom(c(0,0,3),3,prob=parxiEM)+dmultinom(c(1,1,1),3,prob=parxiEM)-(1/m) ) +(1/m) )^2+
				   (- n4)* (3* (dmultinom(c(1,0,1),2,prob=parxiEM)-dmultinom(c(1,1,0),2,prob=parxiEM)-dmultinom(c(0,0,2),2,prob=parxiEM)) ) /
				   (pEM*( dmultinom(c(0,0,3),3,prob=parxiEM)+dmultinom(c(1,1,1),3,prob=parxiEM)-(1/m) ) +(1/m) )+
				   + pEM*n5* (3* (dmultinom(c(1,1,0),2,prob=parxiEM)+dmultinom(c(0,0,2),2,prob=parxiEM)-dmultinom(c(0,1,1),2,prob=parxiEM))*
				   ( dmultinom(c(0,1,2),3,prob=parxiEM)+dmultinom(c(1,2,0),3,prob=parxiEM)-(1/m) ) ) /
				   (pEM*( dmultinom(c(0,1,2),3,prob=parxiEM)+dmultinom(c(1,2,0),3,prob=parxiEM)-(1/m) ) +(1/m) )^2+
				   (- n5)* (3* (dmultinom(c(1,1,0),2,prob=parxiEM)+dmultinom(c(0,0,2),2,prob=parxiEM)-dmultinom(c(0,1,1),2,prob=parxiEM)) ) /
				   (pEM*( dmultinom(c(0,1,2),3,prob=parxiEM)+dmultinom(c(1,2,0),3,prob=parxiEM)-(1/m) ) +(1/m) )+
				   + pEM*n6* (3* (dmultinom(c(0,1,1),2,prob=parxiEM)-dmultinom(c(0,2,0),2,prob=parxiEM))*( dmultinom(c(0,2,1),3,prob=parxiEM)-(1/m) ) ) /
				   (pEM*( dmultinom(c(0,2,1),3,prob=parxiEM)-(1/m) ) +(1/m) )^2+
				   (- n6)* (3* (dmultinom(c(0,1,1),2,prob=parxiEM)-dmultinom(c(0,2,0),2,prob=parxiEM)) ) /
				   (pEM*( dmultinom(c(0,2,1),3,prob=parxiEM)-(1/m) ) +(1/m) )+
				   + pEM*n7* (3*dmultinom(c(0,2,0),2,prob=parxiEM)*( dmultinom(c(0,3,0),3,prob=parxiEM)-(1/m) ) ) /
				   (pEM*( dmultinom(c(0,3,0),3,prob=parxiEM)-(1/m) ) +(1/m) )^2 +
				   (- n7)* ((3*dmultinom(c(0,2,0),2,prob=parxiEM))/
				   (pEM*( dmultinom(c(0,3,0),3,prob=parxiEM)-(1/m) ) +(1/m) ))	
			   	 	

############# I xiD xiU
IxiDxiU <- pEM* (pEM*(-n2)* (9* (dmultinom(c(1,0,1),2,prob=parxiEM)-dmultinom(c(2,0,0),2,prob=parxiEM))* dmultinom(c(2,0,0),2,prob=parxiEM)    ) /
				     (pEM*( dmultinom(c(2,0,1),3,prob=parxiEM)-(1/m) ) +(1/m) )^2
				     + n2* ((6*xDEM)/
				     (pEM*( dmultinom(c(2,0,1),3,prob=parxiEM)-(1/m) ) +(1/m) ))
				     + pEM*n3* (9* (dmultinom(c(1,1,0),2,prob=parxiEM)+dmultinom(c(0,0,2),2,prob=parxiEM)-dmultinom(c(1,0,1),2,prob=parxiEM))*
				     ( dmultinom(c(2,0,0),2,prob=parxiEM)-dmultinom(c(1,0,1),2,prob=parxiEM) ) ) /
				     (pEM*( dmultinom(c(1,0,2),3,prob=parxiEM)+dmultinom(c(2,1,0),3,prob=parxiEM)-(1/m) ) +(1/m) )^2
				  	 + n3* ((6*(1-xUEM-(3*xDEM)))/
				     (pEM*( dmultinom(c(1,0,2),3,prob=parxiEM)+dmultinom(c(2,1,0),3,prob=parxiEM)-(1/m) ) +(1/m) ))
				     + pEM*n4* (9* (dmultinom(c(0,1,1),2,prob=parxiEM)-dmultinom(c(1,1,0),2,prob=parxiEM)-dmultinom(c(0,0,2),2,prob=parxiEM))*
				     ( dmultinom(c(1,0,1),2,prob=parxiEM)-dmultinom(c(1,1,0),2,prob=parxiEM)-dmultinom(c(0,0,2),2,prob=parxiEM) ) ) /
				     (pEM*( dmultinom(c(0,0,3),3,prob=parxiEM)+dmultinom(c(1,1,1),3,prob=parxiEM)-(1/m) ) +(1/m) )^2
				     - n4* ((6*(2-(3*xDEM)-(3*xUEM)))/
				     (pEM*( dmultinom(c(0,0,3),3,prob=parxiEM)+dmultinom(c(1,1,1),3,prob=parxiEM)-(1/m) ) +(1/m) ))
				     + pEM*n5* (9* (dmultinom(c(0,2,0),2,prob=parxiEM)-dmultinom(c(0,1,1),2,prob=parxiEM))*
				     ( dmultinom(c(1,1,0),2,prob=parxiEM)+dmultinom(c(0,0,2),2,prob=parxiEM)-dmultinom(c(0,1,1),2,prob=parxiEM) ) ) /
				     (pEM*( dmultinom(c(0,1,2),3,prob=parxiEM)+dmultinom(c(1,2,0),3,prob=parxiEM)-(1/m) ) +(1/m) )^2
				     + n5* ((6*(1-xDEM-(3*xUEM)))/
				     (pEM*( dmultinom(c(0,1,2),3,prob=parxiEM)+dmultinom(c(1,2,0),3,prob=parxiEM)-(1/m) ) +(1/m) ))
				     - pEM*n6* (9* (dmultinom(c(0,1,1),2,prob=parxiEM)-dmultinom(c(0,2,0),2,prob=parxiEM))* dmultinom(c(0,2,0),2,prob=parxiEM)    ) /
				     (pEM*( dmultinom(c(0,2,1),3,prob=parxiEM)-(1/m) ) +(1/m) )^2
				      + n6* ((6*xUEM)/
				     (pEM*( dmultinom(c(0,2,1),3,prob=parxiEM)-(1/m) ) +(1/m) ))
					)



Imat <- rbind(c(Ipipi,IpixiD,IpixiU),c(IpixiD,IxiDxiD,IxiDxiU),c(IpixiU,IxiDxiU,IxiUxiU))
V <- solve(Imat)

ll <- loglik(c(pEM,xDEM,xUEM),R)

AIC <- 6 - 2*ll
BIC <- 3*log(n) - 2*ll

if(plot==TRUE){
windows()
CUMplot(rp=R,paramp=c(pEM,xDEM,xUEM),m=7)
windows()
print(triCUMplot(param=c(pEM,xDEM,xUEM)))
}

sdxi <- sqrt(V[2,2]+V[3,3]+2*V[2,3])

return(list("pai"=pEM,"xiD"=xDEM,"xiU"=xUEM,"1-xiD-xiU"=1-xDEM-xUEM,"sd"=c(sqrt(diag(V)),sdxi),"Varmat"=V,"Infmat"=Imat,"Diss.index"=diss,"AIC"=AIC,"BIC"=BIC))


}

#######################################################################
##############              log-likelihood            #################
#######################################################################
# inputs:
# param 	     = parameter values c(pai,xiD,xiU)
# data 		     = observed ratings

# output: value of the log-likelihood for given data and parameter values

loglik <- function(param,data) {
	pai <- param[1]
	xiD <- param[2]
	xiU <- param[3]

	tabr<-as.matrix(table(data))
		if(length(tabr) < 7){
		rr<-matrix(0,7,1)
		rr[as.integer(row.names(tabr)),]<-tabr
		tabr<-rr
		}

	probs <- array(NA,7)

	probs[1] <- dmultinom(c(3,0,0),3,prob=c(xiD,xiU,1-xiD-xiU))
	probs[2] <- dmultinom(c(2,0,1),3,prob=c(xiD,xiU,1-xiD-xiU))
	probs[3] <- dmultinom(c(2,1,0),3,prob=c(xiD,xiU,1-xiD-xiU)) +
			dmultinom(c(1,0,2),3,prob=c(xiD,xiU,1-xiD-xiU))
	probs[4] <- dmultinom(c(1,1,1),3,prob=c(xiD,xiU,1-xiD-xiU)) + 
			dmultinom(c(0,0,3),3,prob=c(xiD,xiU,1-xiD-xiU))
	probs[5] <- dmultinom(c(1,2,0),3,prob=c(xiD,xiU,1-xiD-xiU)) +
			dmultinom(c(0,1,2),3,prob=c(xiD,xiU,1-xiD-xiU))
	probs[6] <- dmultinom(c(0,2,1),3,prob=c(xiD,xiU,1-xiD-xiU))
	probs[7] <- dmultinom(c(0,3,0),3,prob=c(xiD,xiU,1-xiD-xiU))

	ll <- sum((log(pai*probs+(1-pai)*(1/7)))%*%tabr)
	ll
}

#######################################################################
##############              NM optimization           #################
#######################################################################
# inputs:
# estEM	 	     = the object generated by the function EM7
# R 		     = observed ratings 


# output: model estimation with NM optimization, with starting values given by the estimates obtained with the EM algorithm
# Note: NM optimization without optimal setting of starting values is very unreliable
# (no convergence, convergence on the boundaries, errors during iterating, ...)

NM7 <- function(estEM,R){

est <- maxLik(loglik,start=c(estEM$pai,estEM$xiD,estEM$xiU),data=R)

return(est)

}


