########################################################################
# Code for examples in the slides
#
## " A mixture model for ordinal variables measured on semantic differential scales "
##   presented at EcoSta 2022 (Kyoto, June 4-6, 2022)
#
# Authors: Paola Zuccolotto & Marica Manisera
# Email: paola.zuccolotto@unibs.it - marica.manisera@unibs.it
# Release: 1.0
# Date: March 2022
# required libraries: maxLik - CUB - ggtern - mc2d - rootSolve
# install.packages("maxLik")
# install.packages("CUB")
# install.packages("ggtern")
# install.packages("mc2d")
# install.packages("rootSolve")
########################################################################

rm(list=ls())

##### set working directory
dir <- "D:\\myfolder"
setwd(dir)

##### libraries and source functions
library(CUB)
source("NLCUB.R")
source("CUM7.R")


################################################
###     case study on distance teaching      ###
################################################


# Please rate your opinion about the following questions between the two given extreme options

# Q1: Do you think that your learning was improved or worsened by distance teaching?
# (response scale: 1=Much worsened, 4=Neither worsened nor improved, 7=Much improved)

# Q2: Do you think that the solutions arranged for distance teaching should be maintained or abandoned after the COVID-19 pandemic?
# (response scale: 1=We should go completely back to the past, 
#  4=We should select what to maintain and what to abandon,
#  7=We should maintain all the novelties introduced during the pandemic.)

# number of categories of the Likert scale
cat <- 7

##### load data
dataDT <- read.table(file="DT_survey.txt",header=TRUE)


# frequency distributions
par(mfrow=c(1,2))
graf <- barplot(as.vector(table(dataDT[,1])),names.arg=c(1:7),main="Q1")
text(graf,as.vector(table(dataDT[,1])),as.character(table(dataDT[,1])),pos=1)
graf <- barplot(as.vector(table(dataDT[,2])),names.arg=c(1:7),main="Q2")
text(graf,as.vector(table(dataDT[,2])),as.character(table(dataDT[,2])),pos=1)

##### fit CUM models
##### display triangular plot
##### plot observed relative frequencies and CUB fitted probabilities (with diss index)
##### compute BIC
##### comparison to CUB and NLCUB
graphics.off()

param <- matrix(NA,2,3)
BIC.DT <- matrix(NA,2,3)

source("CUM7.R")
cum.Q1 <- EM7(R=dataDT$Q1,plot=T)
BIC.DT[1,3] <- cum.Q1$BIC
param[1,] <- c(cum.Q1$pai,cum.Q1$xiD,cum.Q1$xiU)

graphics.off()
cub.Q1 <- GEM(Formula(dataDT$Q1~0|0|0),family="cub",m=cat)
summary(cub.Q1)
pai.Q1 <- cub.Q1$estimates[1]
csi.Q1 <- cub.Q1$estimates[2]
pr.cub.Q1 <- fitted(cub.Q1)[,1]
OFplot(dataDT$Q1,pr.cub.Q1,name="CUB model - Q1",m=cat)
BIC.DT[1,1] <- cub.Q1$BIC

graphics.off()
source("NLCUB.R")
nlcub.Q1 <- NLCUB(dataDT$Q1,m=cat,freq.table=FALSE,method="NM")
nlpai.Q1 <- nlcub.Q1$pai
mi.Q1 <- nlcub.Q1$mi
pr.nlcub.Q1 <- thfr(nlcub.Q1$pai,nlcub.Q1$csi,nlcub.Q1$g)$Fit
BIC.Q1 <- (cat+2)*log(length(na.omit(dataDT$Q1))) - 2*nlcub.Q1[[1]]$maximum
BIC.DT[1,2] <- BIC.Q1

graphics.off()
source("CUM7.R")
cum.Q2 <- EM7(R=dataDT$Q2,plot=T)
BIC.DT[2,3] <- cum.Q2$BIC
param[2,] <- c(cum.Q2$pai,cum.Q2$xiD,cum.Q2$xiU)

graphics.off()
cub.Q2 <- GEM(Formula(dataDT$Q2~0|0|0),family="cub",m=cat)
summary(cub.Q2)
pai.Q2 <- cub.Q2$estimates[1]
csi.Q2 <- cub.Q2$estimates[2]
pr.cub.Q2 <- fitted(cub.Q2)[,1]
OFplot(dataDT$Q2,pr.cub.Q2,name="CUB model - Q2",m=cat)
BIC.DT[2,1] <- cub.Q2$BIC

graphics.off()
source("NLCUB.R")
nlcub.Q2 <- NLCUB(dataDT$Q2,m=cat,freq.table=FALSE,method="NM")
nlpai.Q2 <- nlcub.Q2$pai
mi.Q2 <- nlcub.Q2$mi
pr.nlcub.Q2 <- thfr(nlcub.Q2$pai,nlcub.Q2$csi,nlcub.Q2$g)$Fit
BIC.Q2 <- (cat+2)*log(length(na.omit(dataDT$Q2))) - 2*nlcub.Q2[[1]]$maximum
BIC.DT[2,2] <- BIC.Q2

##### save pdf files of the plots

pdf(file="barplots-DT.pdf",width=8,height=4,paper="special")
par(mfrow=c(1,2))
graf <- barplot(as.vector(table(dataDT[,1])),names.arg=c(1:7),main="Q1")
text(graf,as.vector(table(dataDT[,1])),as.character(table(dataDT[,1])),pos=1)
graf <- barplot(as.vector(table(dataDT[,2])),names.arg=c(1:7),main="Q2")
text(graf,as.vector(table(dataDT[,2])),as.character(table(dataDT[,2])),pos=1)
dev.off()

pdf(file="triangleplotQ1-CUM.pdf",width=6,height=6,paper="special")
triCUMplot(param=param[1,], nmod=1)
dev.off()

pdf(file="triangleplotQ2-CUM.pdf",width=6,height=6,paper="special")
triCUMplot(param=param[2,], nmod=1)
dev.off()

pdf(file="triangleplot-CUM.pdf",width=6,height=6,paper="special")
triCUMplot(param=param, nmod=2)
dev.off()

pdf(file="OFplots-CUM.pdf",width=10,height=5,paper="special")
par(mfrow=c(1,2))
CUMplot(rp=dataDT$Q1,paramp=param[1,])
CUMplot(rp=dataDT$Q2,paramp=param[2,])
dev.off()

pdf(file="OFplots-comparisonDT.pdf",width=12,height=8,paper="special")
par(mfrow=c(2,3))
OFplot(dataDT$Q1,pr.cub.Q1,name="CUB model - Q1",m=cat)
OFplot(dataDT$Q1,pr.nlcub.Q1,name="NLCUB model - Q1",m=cat)
CUMplot(rp=dataDT$Q1,paramp=param[1,])
OFplot(dataDT$Q2,pr.cub.Q2,name="CUB model - Q2",m=cat)
OFplot(dataDT$Q2,pr.nlcub.Q2,name="NLCUB model - Q2",m=cat)
CUMplot(rp=dataDT$Q2,paramp=param[2,])
dev.off()

BIC.DT <- as.data.frame(BIC.DT)
row.names(BIC.DT) <- names(dataDT)
names(BIC.DT) <- c("CUB","NLCUB","CUM")
BIC.DT

save.image()

